Package: joinpointR
Type: Package
Title: Tidy Tools for Joinpoint Regression Models
Version: 1.0.0
Authors@R: 
    person(
        "Tamara", "Ricardo", , 
        email ="tamararicardo83@gmail.com", 
        role = c("aut", "cre"),
        comment = c(ORCID = "0000-0002-0921-2611"))
Description: Provides tools to fit joinpoint regression models with a log-linear 
    specification by levels of a categorical variable. The package acts as a 
    wrapper around the 'segmented' package, facilitating model fitting, selection, 
    and interpretation. It includes functions to estimate the Annual Percent Change 
    (APC) and the Average Annual Percent Change (AAPC), along with their 95% 
    confidence intervals, and to generate formatted summary tables and plots of results.
License: MIT + file LICENSE
Encoding: UTF-8
Imports: dplyr, purrr, tidyr, tibble, ggplot2, segmented, scales,
        flextable, rlang, stringr, stats, officer, forcats, cols4all
URL: https://github.com/datos-ine/joinpointR
BugReports: https://github.com/datos-ine/joinpointR/issues
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
VignetteBuilder: knitr
Depends: R (>= 4.1.0)
LazyData: true
NeedsCompilation: no
Packaged: 2026-06-10 16:03:54 UTC; user
Author: Tamara Ricardo [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0921-2611>)
Maintainer: Tamara Ricardo <tamararicardo83@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-10 17:10:08 UTC
Built: R 4.5.2; ; 2026-06-10 21:22:19 UTC; unix

# Generated by roxygen2: do not edit by hand

export(get_aapc)
export(get_apc)
export(gg_jpoint)
export(jp_to_ft)
export(model_jp)
export(summary_jp)
importFrom(dplyr,where)
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(utils,head)
importFrom(utils,tail)

# joinpointR
<img src="man/figures/logo.png" align="right" width="150" />

<!-- badges: start -->
<!-- badges: end -->

## 🇪🇸 Español

El objetivo de **joinpointR** es ajustar modelos de regresión *joinpoint* por grupos y generar resúmenes en formato *tidy* del Cambio Porcentual Anual (APC) y del Cambio Porcentual Anual Promedio (AAPC), facilitando el análisis de tendencias en estudios epidemiológicos.

---

## 🇬🇧 English

The goal of **joinpointR** is to fit *joinpoint regression models* by groups and generate tidy summaries of the Annual Percent Change (APC) and the Average Annual Percent Change (AAPC), facilitating trend analysis in epidemiological studies.

---

## Installation / Instalación

You can install the development version from GitHub / Podés instalar la versión en desarrollo desde GitHub:

```r
# install.packages("pak")
pak::pak("datos-ine/joinpointR")
```

Repository: https://github.com/datos-ine/joinpointR

## Workflow / Flujo de trabajo

The package provides a simple and reproducible workflow / El paquete propone un flujo simple y reproducible:

* Fit joinpoint models by group / Ajustar modelos joinpoint por grupo
* Extract APC by segment / Extraer APC por segmento
* Compute AAPC / Calcular AAPC
* Generate summary tables / Generar tablas resumen

## Main functions / Funciones principales
* `model_jp()` → fit joinpoint models by group / ajusta modelos joinpoint por grupo
* `get_apc()` → extract APC by segment / extrae APC por segmento
* `get_aapc()` → compute AAPC / computa AAPC
* `summary_jp()` → generate summary tables (tibble or flextable) / genera tablas resumen (tibble o flextable)

## Example / Ejemplo
```r
library(joinpointR)
library(dplyr)

df <- tibble(
  year = rep(2000:2010, 2),
  rate = c(runif(11, 10, 20), runif(11, 5, 15)),
  group = rep(c("Male", "Female"), each = 11)
)

mods <- model_jp(
  data = df,
  value = "rate",
  time = "year",
  group = "group",
  step = TRUE
)

# APC (only works when class segmented lm)
get_apc(mods$Male, digits = 1, time = "year", dec = ".") # Will generate an error

get_apc(mods$Female, digits = 1, time = "year", dec = ".")

# AAPC with 95% CI
get_aapc(mods$Male, show_ci = TRUE)

# AAPC with significance stars
get_aapc(mods$Male, show_ci = FALSE)

# Summary Table
summary_jp(mods)
```

### Formatted table / Tabla formateada
```r
# English (default)
summary_jp(mods, ft = TRUE, lan = "en", var1 = "Sex")

# Spanish
summary_jp(mods, ft = TRUE, lan = "es", var1 = "Sexo")

```

Returns a table ready for reporting (e.g., Word) using flextable. / Devuelve una tabla lista para exportar a Word o informes mediante `flextable`.

### Output/Salida

The generated table includes / La tabla generada incluye:

* Number of joinpoints / Número de joinpoints (JP)
* Time periods for each segment / Períodos de cada segmento
* APC per segment / por segmento
* 95% Confidence intervals / Intervalos de confianza al 95%
* AAPC (global tendency / tendencia global)

### Plots / Gráficos
```r
# Plot results
mods |>
  gg_joinpoint(obs = TRUE, jp = TRUE, facets = FALSE)

# Facets by group
mods |>
  gg_jpoint(obs = TRUE, jp = TRUE, facets = TRUE)

# Hide observed
mods |>
  gg_jpoint(obs = FALSE, jp = TRUE, facets = FALSE)

# Hide joinpoints
mods |>
  gg_jpoint(obs = TRUE, jp = FALSE, facets = FALSE)
```

## Dependencies / Dependencias
The package uses / El paquete utiliza:

* `segmented` for fitting jointpoint regression models / para regresión joinpoint
* `dplyr`, `purrr`, `tidyr`, `tibble` for data management / para manipulación de datos
* `ggplot2` for plotting results / para graficar resultados
* `flextable` for summary tables / para tablas formateadas

## Notes / Notas
* The response variable is log-transformed / La variable respuesta se transforma logarítmicamente
* Model selection is based on the Bayesian Information Criterion (BIC) / La selección de modelos se basa en el Criterio de Información Bayesiano (BIC)
* When `step = FALSE` fits a joinpoint regression model with the number of joinpoints specified in `k`/ Cuando `step = FALSE` ajusta una regresión joinpoint para el número de joinpoints especificados en `k`.
* Results are returned in tidy format / Los resultados se devuelven en formato tidy para facilitar su uso en análisis reproducibles
* Formatted tables are optional / La creación de tablas formateadas es opcional `(ft = TRUE)`

## Licence / Licencia
MIT License

## Author / Autora
Tamara Ricardo
Instituto Nacional de Epidemiología (INE), Argentina
ORCID: https://orcid.org/0000-0002-0921-2611

# joinpointR

## 🇪🇸 Español

El objetivo de **joinpointR** es ajustar modelos de regresión *joinpoint* por grupos y generar resúmenes en formato *tidy* del Cambio Porcentual Anual (APC) y del Cambio Porcentual Anual Promedio (AAPC), facilitando el análisis de tendencias en estudios epidemiológicos.

---

## 🇬🇧 English

The goal of **joinpointR** is to fit *joinpoint regression models* by groups and generate tidy summaries of the Annual Percent Change (APC) and the Average Annual Percent Change (AAPC), facilitating trend analysis in epidemiological studies.

---

## Installation / Instalación

You can install the development version from GitHub / Podés instalar la versión en desarrollo desde GitHub:

```r
# install.packages("pak")
pak::pak("datos-ine/joinpointR")
```

Repository: https://github.com/datos-ine/joinpointR

## Workflow / Flujo de trabajo

The package provides a simple and reproducible workflow / El paquete propone un flujo simple y reproducible:

* Fit joinpoint models by group / Ajustar modelos joinpoint por grupo
* Extract APC by segment / Extraer APC por segmento
* Compute AAPC / Calcular AAPC
* Generate summary tables / Generar tablas resumen

## Main functions / Funciones principales
* `model_jp()` → fit joinpoint models by group / ajusta modelos joinpoint por grupo
* `get_apc()` → extract APC by segment / extrae APC por segmento
* `get_aapc()` → compute AAPC / computa AAPC
* `summary_jp()` → generate summary tables (tibble or flextable) / genera tablas resumen (tibble o flextable)

## Example / Ejemplo
```r
library(joinpointR)
library(dplyr)

df <- tibble(
  year = rep(2000:2010, 2),
  rate = c(runif(11, 10, 20), runif(11, 5, 15)),
  group = rep(c("Male", "Female"), each = 11)
)

mods <- model_jp(
  data = df,
  value = "rate",
  time = "year",
  group = "group",
  step = TRUE
)

# APC (only works when class segmented lm)
get_apc(mods$Male, digits = 1, time = "year", dec = ".") # Will generate an error

get_apc(mods$Female, digits = 1, time = "year", dec = ".")

# AAPC with 95% CI
get_aapc(mods$Male, show_ci = TRUE)

# AAPC with significance stars
get_aapc(mods$Male, show_ci = FALSE)

# Summary Table
summary_jp(mods)
```

### Formatted table / Tabla formateada
```r
# English (default)
summary_jp(mods, ft = TRUE, lan = "en", var1 = "Sex")

# Spanish
summary_jp(mods, ft = TRUE, lan = "es", var1 = "Sexo")

```

Returns a table ready for reporting (e.g., Word) using flextable. / Devuelve una tabla lista para exportar a Word o informes mediante `flextable`.

### Output/Salida

The generated table includes / La tabla generada incluye:

* Number of joinpoints / Número de joinpoints (JP)
* Time periods for each segment / Períodos de cada segmento
* APC per segment / por segmento
* 95% Confidence intervals / Intervalos de confianza al 95%
* AAPC (global tendency / tendencia global)

### Plots / Gráficos
```r
# Plot results
mods |>
  gg_joinpoint(obs = TRUE, jp = TRUE, facets = FALSE)

# Facets by group
mods |>
  gg_jpoint(obs = TRUE, jp = TRUE, facets = TRUE)

# Hide observed
mods |>
  gg_jpoint(obs = FALSE, jp = TRUE, facets = FALSE)

# Hide joinpoints
mods |>
  gg_jpoint(obs = TRUE, jp = FALSE, facets = FALSE)
```

## Dependencies / Dependencias
The package uses / El paquete utiliza:

* `segmented` for fitting jointpoint regression models / para regresión joinpoint
* `dplyr`, `purrr`, `tidyr`, `tibble` for data management / para manipulación de datos
* `ggplot2` for plotting results / para graficar resultados
* `flextable` for summary tables / para tablas formateadas

## Notes / Notas
* The response variable is log-transformed / La variable respuesta se transforma logarítmicamente
* Model selection is based on the Bayesian Information Criterion (BIC) / La selección de modelos se basa en el Criterio de Información Bayesiano (BIC)
* When `step = FALSE` fits a joinpoint regression model with the number of joinpoints specified in `k`/ Cuando `step = FALSE` ajusta una regresión joinpoint para el número de joinpoints especificados en `k`.
* Results are returned in tidy format / Los resultados se devuelven en formato tidy para facilitar su uso en análisis reproducibles
* Formatted tables are optional / La creación de tablas formateadas es opcional `(ft = TRUE)`

## Licence / Licencia
MIT License

## Author / Autora
Tamara Ricardo
Instituto Nacional de Epidemiología (INE), Argentina
ORCID: https://orcid.org/0000-0002-0921-2611

#' @importFrom rlang .data :=
NULL

utils::globalVariables(
  c(
    "APC",
    "CI",
    "CI_l",
    "CI_u",
    "lci",
    "uci",
    "Period",
    "group",
    "obs",
    "jp",
    "facets",
    "fit"
  )
)

#' @importFrom dplyr where
#' @importFrom utils head tail
NULL

#' Plot Joinpoint Regression Models
#'
#' Creates a ggplot showing observed values, fitted joinpoint regression lines,
#' and optional joinpoints.
#'
#' @param mods List of joinpoint regression models (output of model_jp()).
#' @param obs Logical. If TRUE, displays observed data points.
#' @param jp Logical. If TRUE, displays joinpoints as vertical dashed lines.
#' @param facets Logical. If TRUE, displays one panel per group using facets.
#'
#' @return A ggplot object.
#' @export
#' @examples
#' library(dplyr)
#' # Load example data
#' data("plant", package = "segmented")
#'
#' names(plant)
#'
#' # Fit the joinpoint models
#' mods <- model_jp(data = plant, value = "y", time = "time", group = "group", k = 2, test = TRUE)
#'
#' # Plot results
#' gg_jpoint(mods, obs = TRUE, jp = TRUE, facets = FALSE)
#'
#' # Facets by group
#' gg_jpoint(mods, obs = TRUE, jp = TRUE, facets = TRUE)

gg_jpoint <- function(mods, obs = TRUE, jp = TRUE, facets = FALSE) {
  # ---- Detect time variable ----
  time <- names(mods[[1]]$model)[2]

  # ---- Plot data ----
  df <- purrr::map_df(
    mods,
    ~ tibble::tibble(
      time = .x$model[[time]],
      obs = .x$model[[1]],
      fit = stats::predict(.x)
    ),
    .id = "group"
  )

  # ---- Joinpoints ----
  jp_df <- purrr::map_df(
    mods,
    ~ {
      if (inherits(.x, "segmented") && !is.null(.x$psi)) {
        tibble::tibble(jp = .x$psi[, "Est."])
      } else {
        NULL
      }
    },
    .id = "group"
  )

  # ---- Base plot ----
  p <- ggplot2::ggplot(df, ggplot2::aes(x = time, y = obs, color = group)) +
    ggplot2::geom_line(ggplot2::aes(y = fit), linewidth = 1)

  # ---- Observaciones ----
  if (obs) {
    p <- p + ggplot2::geom_point(size = 2.5, alpha = 0.75)
  }

  # ---- Joinpoints ----
  if (jp && nrow(jp_df) > 0) {
    p <- p +
      ggplot2::geom_vline(
        data = jp_df,
        ggplot2::aes(xintercept = jp),
        color = "darkgrey",
        lwd = 2,
        alpha = 0.5
      )
  }

  # ---- Facets ----
  if (facets) {
    p <- p + ggplot2::facet_wrap(~group)
  }

  # ---- Final ----
  p +
    ggplot2::labs(
      x = NULL,
      y = "log(rate)"
    ) +
    ggplot2::theme_minimal()
}

#' Average Annual Percent Change (AAPC)
#'
#' Estimates the Average Annual Percent Change (AAPC) and its 95% confidence
#' interval. Optionally displays statistical significance using significance
#' stars.
#'
#' @param mod Joinpoint regression model (segmented object) or linear regression model (lm object).
#' @param digits Number of decimal places to display (integer).
#' @param show_ci Logical; if TRUE, displays the 95% confidence interval.
#'   If FALSE, displays significance stars.
#' @param dec Character. Decimal separator to use (e.g., "." or ",").
#'
#' @return A character string with the AAPC and either its 95% confidence
#' interval or significance stars.
#' @author Tamara Ricardo
#' @export
#'
#' @examples
#' # Load example data
#' data("plant", package = "segmented")
#'
#' names(plant)
#'
#' # Fit the joinpoint models
#' mods <- model_jp(data = plant, value = "y", time = "time", group = "group", k = 2, test = TRUE)
#'
#' # AAPC of the first model
#' get_aapc(mods$RKW, digits = 1, show_ci = TRUE, dec = ".")

get_aapc <- function(mod, digits = 1, show_ci = TRUE, dec = ".") {
  # ---- Validations ----
  if (!inherits(mod, c("segmented", "lm"))) {
    stop("`mod` must be an object of class 'segmented' or 'lm'")
  }

  # ---- Helpers ----
  fmt_ci <- function(x, y, z) {
    paste0(
      scales::percent(x, accuracy = 10^-digits, decimal.mark = dec),
      " (",
      scales::percent(y, accuracy = 10^-digits, decimal.mark = dec),
      "; ",
      scales::percent(z, accuracy = 10^-digits, decimal.mark = dec),
      ")"
    )
  }

  fmt_stars <- function(x, stars) {
    paste0(
      scales::percent(x, accuracy = 10^-digits, decimal.mark = dec),
      ifelse(stars != "", paste0(" ", stars), "")
    )
  }

  # ---- Calculations ----
  if (inherits(mod, "segmented")) {
    aapc_obj <- segmented::aapc(mod)

    est <- unname(aapc_obj[grep("Est", names(aapc_obj))])
    lci <- unname(aapc_obj[grep("\\.l", names(aapc_obj))])
    uci <- unname(aapc_obj[grep("\\.u", names(aapc_obj))])
  } else {
    beta <- stats::coef(mod)[2]

    est <- exp(beta) - 1
    ci <- stats::confint(mod)[2, ]

    lci <- exp(ci[1]) - 1
    uci <- exp(ci[2]) - 1
  }

  # ---- Significance based on CI ----
  stars <- ifelse(lci > 0 | uci < 0, "*", "")

  # ---- Output ----
  if (show_ci) {
    fmt_ci(est, lci, uci)
  } else {
    fmt_stars(est, stars)
  }
}

#' Summary Table For Joinpoint Regression Models
#'
#' Generates a table displaying the number of joinpoints, time breaks, APC and its 95% confidence
#' interval, and AAPC and its statistical significance from a list of joinpoint models.
#'
#' @param mods List of joinpoint regression models (model_jp() output).
#' @param digits Number of decimal places to display (integer).
#' @param var1 Character. Name of the grouping variable.
#' @param var2 Character. Name of the subgrouping variable (optional).
#' @param ft Logical. If TRUE returns a flextable object, if FALSE returns a tibble.
#' @param lan Language of output: "en" (English) or "es" (Spanish).
#'
#' @return A tibble or a flextable object.
#' @author Tamara Ricardo
#' @export
#' @examples
#' library(dplyr)
#' # Load example data
#' data("plant", package = "segmented")
#'
#' names(plant)
#'
#' # Fit the joinpoint models
#' mods <- model_jp(data = plant, value = "y", time = "time", group = "group", k = 2, test = TRUE)
#'
#' summary_jp(mods, digits = 1, var1 = "group", ft = FALSE, lan = "en")

summary_jp <- function(
  mods,
  digits = 1,
  var1 = "group",
  var2 = "subgroup",
  ft = FALSE,
  lan = c("en", "es")
) {
  lan <- match.arg(lan)
  dec <- if (lan == "en") "." else ","

  df <- purrr::map_dfr(
    mods,
    function(mod) {
      if (inherits(mod, "segmented")) {
        time <- names(mod$model)[2]

        breaks <- sort(c(
          min(mod$model[[time]]),
          mod$psi[, "Est."],
          max(mod$model[[time]])
        ))

        n_jp <- nrow(mod$psi)

        apc_tbl <- segmented::slope(mod, APC = TRUE)[[time]] |>
          tibble::as_tibble() |>
          dplyr::rename(APC = 1, CI_l = 2, CI_u = 3) |>
          dplyr::mutate(
            dplyr::across(where(is.numeric), ~ round(.x, digits)),
            JP = ifelse(dplyr::row_number() == 1, n_jp, NA_real_),
            Period = paste(
              round(head(breaks, -1)),
              round(tail(breaks, -1)),
              sep = "-"
            ),
            .before = APC
          ) |>
          dplyr::mutate(
            AAPC = ifelse(
              dplyr::row_number() == 1,
              get_aapc(mod, show_ci = FALSE, dec = dec),
              NA_character_
            )
          ) |>
          tidyr::unite("CI", CI_l, CI_u, sep = "; ")

        apc_tbl
      } else {
        tibble::tibble(
          JP = 0,
          Period = NA_character_,
          APC = NA_real_,
          CI = NA_character_,
          AAPC = get_aapc(mod, show_ci = FALSE, dec = dec)
        )
      }
    },
    .id = "group"
  )

  # ---- split grouping ----
  df <- if (any(stringr::str_detect(df$group, "_"))) {
    tidyr::separate_wider_delim(
      df,
      group,
      names = c(var1, var2),
      delim = "_",
      too_few = "align_start"
    )
  } else {
    dplyr::rename(df, !!var1 := group)
  }

  # ---- language ----
  if (lan == "es") {
    df <- df |>
      dplyr::rename(
        Periodo = Period,
        IC = CI
      )

    footnote_txt <- paste0(
      "* p < 0,05\n",
      "JP: cantidad de joinpoints; APC: cambio porcentual anual; ",
      "IC: intervalo de confianza al 95%; AAPC: cambio porcentual anual promedio (IC95%)."
    )
  } else {
    footnote_txt <- paste0(
      "* p < 0.05\n",
      "JP: number of joinpoints; APC: annual percent change; ",
      "CI: 95% confidence interval; AAPC: average annual percent change (95% CI)."
    )
  }

  if (!ft) {
    return(df)
  }

  # ---- flextable ----
  ft_obj <- flextable::flextable(df)

  cols_merge <- intersect(c(var1, var2), names(df))

  if (var2 %in% names(df) && all(is.na(df[[var2]]))) {
    ft_obj <- flextable::delete_columns(ft_obj, var2)
    cols_merge <- var1
  }

  ft_obj |>
    flextable::merge_v(j = cols_merge) |>
    flextable::bold(part = "header") |>
    flextable::colformat_num(
      j = names(df)[sapply(df, is.numeric)],
      decimal.mark = dec,
      big.mark = ""
    ) |>
    flextable::add_body_row(
      top = FALSE,
      values = list(footnote_txt),
      colwidths = ncol(df)
    ) |>
    flextable::hline_bottom(border = officer::fp_border(width = 0)) |>
    flextable::font(fontname = "Calibri", part = "all") |>
    flextable::fontsize(size = 12, part = "all") |>
    flextable::align(align = "left", part = "all") |>
    flextable::autofit()
}

#' Annual Percent Change by Segment (APC)
#'
#' Calculates the Annual Percent Change (APC) for each segment and its 95%
#' confidence interval.
#'
#' @param mod Joinpoint regression model (segmented object).
#' @param digits Number of decimal places to display (integer).
#' @param time Time variable used in the model (character).
#' @param dec Character. Decimal separator to use (e.g., "." or ",").
#'
#' @return A character vector with APC and 95% CI for each segment.
#' @author Tamara Ricardo
#' @export
#'
#' @examples
#' # Load example data
#' data("plant", package = "segmented")
#'
#' names(plant)
#'
#' # Fit the joinpoint models
#' mods <- model_jp(data = plant, value = "y", time = "time", group = "group", k = 2, test = TRUE)
#'
#' # Obtain APC (95% CI)
#' get_apc(mods$RKW, digits = 1, time = "time", dec = ".")

get_apc <- function(mod, digits = 1, time = "time", dec = ".") {
  # ---- Validation ----
  if (!inherits(mod, "segmented")) {
    stop("`mod` must be an object of class 'segmented'")
  }

  # ---- APC ----
  segmented::slope(mod, APC = TRUE)[1] |>
    data.frame() |>
    dplyr::rename(APC = 1, CI_l = 2, CI_u = 3) |>
    dplyr::mutate(
      dplyr::across(
        where(is.numeric),
        ~ scales::number(
          .x,
          accuracy = 10^-digits,
          decimal.mark = dec,
          suffix = "%"
        )
      )
    ) |>
    tidyr::unite("CI", CI_l, CI_u, sep = "; ")
}

#' Joinpoint Regression Models by Groups
#'
#' Fits segmented linear regression models by groups for age-standardized rates
#' using joinpoint regression. Models can be fitted using either a stepwise
#' selection procedure based on the Bayesian Information Criterion (BIC) or a
#' fixed number of joinpoints. Internally calls
#' \code{segmented::selgmented()} or \code{segmented::segmented()} and applies
#' a log transformation to the response variable.
#'
#' @param data Data frame containing age-standardized rates.
#' @param value Name of the response variable (character).
#' @param time Name of the time variable (character).
#' @param group Name of one or more grouping variables (character vector).
#' @param k Maximum number of joinpoints to be estimated (integer).
#' @param step Use stepwise procedure to select the number of joinpoints (logical).
#' @param test whether to test for differences in slope t-values during the stepwise selection
#' procedure. Only used when \code{step = TRUE}.
#' @return A list of models by group.
#' @author Tamara Ricardo
#' @export
#'
#' @examples
#' # Generate example data
#' library(dplyr)
#' df <- mtcars |>
#' mutate(
#'   year = seq(2000, length.out = n(), by = 1),
#'   group = factor(paste("cyl", cyl, sep = "_")),
#'   rate = mpg
#' ) |>
#' select(year, group, rate)
#'
#' # Check group levels
#' levels(df$group)
#'
#' # Fit models
#' mods <- model_jp(data = df, value = "rate", time = "year", group = "group",
#'  k = 2, step = TRUE, test = TRUE)
#'
#' # Show the output of the first model
#' mods$cyl_6
#' summary(mods$cyl_6)

model_jp <- function(
  data,
  value,
  time,
  group,
  k = 2,
  step = TRUE,
  test = TRUE
) {
  # ---- Validate response variable ----
  if (any(data[[value]] <= 0, na.rm = TRUE)) {
    stop("The response variable must be > 0 to apply log() transformation")
  }

  # ---- Validate grouping variable/s ----
  if (!all(group %in% names(data))) {
    stop("Some grouping variables are not present in data")
  }

  # ---- Validate test and step arguments ----
  if (!step && test) {
    warning("'test' is ignored when step = FALSE")
  }

  # ---- Prepare data ----
  data <- data |>
    # Transform the response variable
    dplyr::mutate(
      .jp_log_value = log(.data[[value]])
    ) |>

    # Create grouping variable
    dplyr::mutate(
      .jp_group = forcats::fct_drop(
        forcats::fct_cross(
          !!!rlang::syms(group),
          sep = "_"
        )
      )
    ) |>

    # Group data
    dplyr::group_by(.data$.jp_group)

  # ---- Formula ----
  formula <- stats::reformulate(
    termlabels = time,
    response = ".jp_log_value"
  )

  # ---- Group names ----
  groups <- data |>
    dplyr::distinct(.data$.jp_group) |>
    dplyr::pull(.data$.jp_group)

  # ---- Model fit ----
  if (step) {
    mods <- data |>
      dplyr::group_map(
        ~ segmented::selgmented(
          olm = stats::lm(formula, data = .x),
          Kmax = k,
          type = "bic",
          th = 2,
          stop.if = 4,
          check.dslope = test
        )
      )
  } else {
    mods <- data |>
      dplyr::group_map(
        ~ segmented::segmented(
          obj = stats::lm(formula, data = .x),
          seg.Z = stats::as.formula(
            paste0("~", time)
          ),
          npsi = k
        )
      )
  }

  # ---- Name models ----
  rlang::set_names(mods, groups)
}

---
title: "Introduction"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false
#| id: settings
knitr::opts_chunk$set(
  warning = FALSE,
  collapse = TRUE,
  comment = "#>"
)
```

Joinpoint regression is commonly used in epidemiology to identify changes in temporal trends. The joinpointR package provides tools to fit joinpoint regression models, estimate annual percentage changes, and present results.

## Joinpoint regression models by group
The function `model_jp()`allows to fit joinpoint regression models with a log-response by levels of one or more categorical variable(s).
For the examples, we are going to use a simulated dataset with the HIV rates by sex in five regions

```{r}
#| message: false
#| id: example-data
# Load required packages
library(dplyr)
library(tidyr)
library(ggplot2)
library(joinpointR)

# Load example data
data(hiv_data)
```

### Stepwise joinpoint regression by sex
```{r}
#| id: model-1
mod1 <- model_jp(
  data = hiv_data,
  value = hiv_rate,
  time = year,
  group = "sex",
  k = 3,
  step = TRUE,
  test = TRUE
)

# Show the model output
mod1
```

### Fixed number of joinpoints by sex
```{r}
#| id: model-2
mod2 <- model_jp(
  data = hiv_data,
  value = hiv_rate,
  time = year,
  group = "sex",
  k = 1,
  step = FALSE,
  test = FALSE
)

# Show the model output
mod2
```

### Stepwise joinpoint regression models by sex and region
```{r}
#| id: model-3
mod3 <- model_jp(
  data = hiv_data,
  value = hiv_rate,
  time = year,
  group = c("region", "sex"),
  step = TRUE
)
```

## Annual percentage change (APC)
The function `get_apc()` provides the annual percentage change and its 95% confidence interval for models with significant joinpoints (`class: "segmented" "lm"`)

```{r}
#| id: APC-1
get_apc(mod2)
```

The function returns empty rows when no significant joinpoints are detected (`class = "lm"`).

```{r}
#| id: APC-2
get_apc(mod1, digits = 1)
```


## Average annual percent change (AAPC)
The function `get_aapc()` allows users to estimate the global trend even when no significant joinpoints were detected.

```{r}
#| id: AAPC-1
get_aapc(mod1, digits = 1, show_ci = TRUE)
```

You can choose to show significance stars instead of CI:
```{r}
#| id: AAPC-2
get_aapc(mod1, digits = 1, show_ci = FALSE)
```

## Summary tables
### Results as a data frame
```{r}
#| id: tab-1
summary_jp(mod1)
```

### HTML Tables
```{r}
# id: tab-2
mod2 |>
  summary_jp(digits = 1) |>
  jp_to_ft()
```

### HTML Tables by two grouping variables
```{r}
#| id: tab-3
mod3 |>
  summary_jp(digits = 1) |>
  jp_to_ft()
```

Change the table language to Spanish
```{r}
#| id: tab-4
mod3 |>
  summary_jp(digits = 1) |>
  jp_to_ft(lan = "es")
```

## Regression plots
Default plot
```{r}
#| id: plot-1
gg_jpoint(mod1)
```

Stack plots
```{r}
#| id: plot-2
gg_jpoint(mod1, facets = "none")
```

Two grouping variables

```{r}
#| id: plot-3
gg_jpoint(mod3)
```

Split by region and sex
```{r}
#| id: plot-4
gg_jpoint(mod3, facets = "grid")
```

Modify the size of data points
```{r}
#| id: plot-5
gg_jpoint(mod3, facets = "grid", psize = 5)
```

Modify the transparency of data points
```{r}
#| id: plot-6
gg_jpoint(mod3, facets = "grid", ptr = 1)
```

Hide data points
```{r}
#| id: plot-7
gg_jpoint(mod3, facets = "grid", obs = FALSE)
```

Hide joinpoint(s) marker(s)
```{r}
#| id: plot-8
gg_jpoint(mod3, facets = "grid", jp = FALSE)
```

Change the default colorblind-friendly palette
```{r}
#| id: plot-9
gg_jpoint(mod3, facets = "grid", jp = FALSE, cbpal = "blue_fluoride")
```

Disable colorblind-friendly palettes
```{r}
#| id: plot- 10
gg_jpoint(mod3, facets = "grid", cb = FALSE)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_jp.R
\name{model_jp}
\alias{model_jp}
\title{Joinpoint Regression Models by Groups}
\usage{
model_jp(data, value, time, group, k = 2, step = TRUE, test = TRUE)
}
\arguments{
\item{data}{Data frame containing age-standardized rates.}

\item{value}{Name of the response variable (character).}

\item{time}{Name of the time variable (character).}

\item{group}{Name of one or more grouping variables (character vector).}

\item{k}{Maximum number of joinpoints to be estimated (integer).}

\item{step}{Use stepwise procedure to select the number of joinpoints (logical).}

\item{test}{whether to test for differences in slope t-values during the stepwise selection
procedure. Only used when \code{step = TRUE}.}
}
\value{
A list of models by group.
}
\description{
Fits segmented linear regression models by groups for age-standardized rates
using joinpoint regression. Models can be fitted using either a stepwise
selection procedure based on the Bayesian Information Criterion (BIC) or a
fixed number of joinpoints. Internally calls
\code{segmented::selgmented()} or \code{segmented::segmented()} and applies
a log transformation to the response variable.
}
\examples{
# Generate example data
library(dplyr)
df <- mtcars |>
mutate(
  year = seq(2000, length.out = n(), by = 1),
  group = factor(paste("cyl", cyl, sep = "_")),
  rate = mpg
) |>
select(year, group, rate)

# Check group levels
levels(df$group)

# Fit models
mods <- model_jp(data = df, value = "rate", time = "year", group = "group",
 k = 2, step = TRUE, test = TRUE)

# Show the output of the first model
mods$cyl_6
summary(mods$cyl_6)
}
\author{
Tamara Ricardo
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_aapc.R
\name{get_aapc}
\alias{get_aapc}
\title{Average Annual Percent Change (AAPC)}
\usage{
get_aapc(mod, digits = 1, show_ci = TRUE, dec = ".")
}
\arguments{
\item{mod}{Joinpoint regression model (segmented object) or linear regression model (lm object).}

\item{digits}{Number of decimal places to display (integer).}

\item{show_ci}{Logical; if TRUE, displays the 95\% confidence interval.
If FALSE, displays significance stars.}

\item{dec}{Character. Decimal separator to use (e.g., "." or ",").}
}
\value{
A character string with the AAPC and either its 95\% confidence
interval or significance stars.
}
\description{
Estimates the Average Annual Percent Change (AAPC) and its 95\% confidence
interval. Optionally displays statistical significance using significance
stars.
}
\examples{
# Load example data
data("plant", package = "segmented")

names(plant)

# Fit the joinpoint models
mods <- model_jp(data = plant, value = "y", time = "time", group = "group", k = 2, test = TRUE)

# AAPC of the first model
get_aapc(mods$RKW, digits = 1, show_ci = TRUE, dec = ".")
}
\author{
Tamara Ricardo
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary_jp.R
\name{summary_jp}
\alias{summary_jp}
\title{Summary Table For Joinpoint Regression Models}
\usage{
summary_jp(
  mods,
  digits = 1,
  var1 = "group",
  var2 = "subgroup",
  ft = FALSE,
  lan = c("en", "es")
)
}
\arguments{
\item{mods}{List of joinpoint regression models (model_jp() output).}

\item{digits}{Number of decimal places to display (integer).}

\item{var1}{Character. Name of the grouping variable.}

\item{var2}{Character. Name of the subgrouping variable (optional).}

\item{ft}{Logical. If TRUE returns a flextable object, if FALSE returns a tibble.}

\item{lan}{Language of output: "en" (English) or "es" (Spanish).}
}
\value{
A tibble or a flextable object.
}
\description{
Generates a table displaying the number of joinpoints, time breaks, APC and its 95\% confidence
interval, and AAPC and its statistical significance from a list of joinpoint models.
}
\examples{
library(dplyr)
# Load example data
data("plant", package = "segmented")

names(plant)

# Fit the joinpoint models
mods <- model_jp(data = plant, value = "y", time = "time", group = "group", k = 2, test = TRUE)

summary_jp(mods, digits = 1, var1 = "group", ft = FALSE, lan = "en")
}
\author{
Tamara Ricardo
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_apc.R
\name{get_apc}
\alias{get_apc}
\title{Annual Percent Change by Segment (APC)}
\usage{
get_apc(mod, digits = 1, time = "time", dec = ".")
}
\arguments{
\item{mod}{Joinpoint regression model (segmented object).}

\item{digits}{Number of decimal places to display (integer).}

\item{time}{Time variable used in the model (character).}

\item{dec}{Character. Decimal separator to use (e.g., "." or ",").}
}
\value{
A character vector with APC and 95\% CI for each segment.
}
\description{
Calculates the Annual Percent Change (APC) for each segment and its 95\%
confidence interval.
}
\examples{
# Load example data
data("plant", package = "segmented")

names(plant)

# Fit the joinpoint models
mods <- model_jp(data = plant, value = "y", time = "time", group = "group", k = 2, test = TRUE)

# Obtain APC (95\% CI)
get_apc(mods$RKW, digits = 1, time = "time", dec = ".")
}
\author{
Tamara Ricardo
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_jpoint.R
\name{gg_jpoint}
\alias{gg_jpoint}
\title{Plot Joinpoint Regression Models}
\usage{
gg_jpoint(mods, obs = TRUE, jp = TRUE, facets = FALSE)
}
\arguments{
\item{mods}{List of joinpoint regression models (output of model_jp()).}

\item{obs}{Logical. If TRUE, displays observed data points.}

\item{jp}{Logical. If TRUE, displays joinpoints as vertical dashed lines.}

\item{facets}{Logical. If TRUE, displays one panel per group using facets.}
}
\value{
A ggplot object.
}
\description{
Creates a ggplot showing observed values, fitted joinpoint regression lines,
and optional joinpoints.
}
\examples{
library(dplyr)
# Load example data
data("plant", package = "segmented")

names(plant)

# Fit the joinpoint models
mods <- model_jp(data = plant, value = "y", time = "time", group = "group", k = 2, test = TRUE)

# Plot results
gg_jpoint(mods, obs = TRUE, jp = TRUE, facets = FALSE)

# Facets by group
gg_jpoint(mods, obs = TRUE, jp = TRUE, facets = TRUE)
}
